<?php

use CleverReach\BusinessLogic\Sync\RefreshUserInfoTask;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\TaskExecution\Queue;
use CleverReach\BusinessLogic\Interfaces\Proxy;
use CleverReach\Infrastructure\ServiceRegister;

/**
 * Callback View Controller.
 *
 * @see template file cleverreach-welcome.tpl.php
 */
class CleverReachCallbackController extends CleverReachResolveStateController {
  const TEMPLATE = 'cleverreach_callback';

  /**
   * Executes GET request method.
   *
   * CleverReach auth callback controller.
   *
   * @param array $form
   *   A keyed array containing the current state of the form.
   *
   * @return array
   *   Array of template vars.
   *
   * @throws Exception
   */
  public function get(array &$form) {
    $params = drupal_get_query_parameters();

    if (empty($params['code'])) {
      drupal_set_message(t('Wrong parameters. Code not set.'));
    }
    else {
      $result = $this->getAccessToken($params['code']);

      if (isset($result['error']) || empty($result['access_token'])) {
        drupal_set_message(
        isset($result['error_description'])
            ? $result['error_description']
            : t('Unsuccessful connection.')
        );
      }
      else {
        try {
          $this->queueRefreshUserTask($result['access_token']);
        }
        catch (Exception $e) {
          drupal_set_message($e->getMessage());
        }
      }
    }

    return array('#theme' => self::TEMPLATE);
  }

  /**
   * Gets CleverReach access token.
   *
   * @param string $code
   *   Code retrieved from CleverReach.
   *
   * @return array
   *   Access token array.
   */
  private function getAccessToken($code) {
    /** @var CleverReach\BusinessLogic\Proxy $proxy */
    $proxy = ServiceRegister::getService(Proxy::CLASS_NAME);

    return $proxy->getAccessToken($code, $this->getControllerUrl('callback'));
  }

  /**
   * Enqueues refresh user task.
   *
   * @param string $accessToken
   *   CleverReach access token.
   */
  private function queueRefreshUserTask($accessToken) {
    /** @var CleverReach\Infrastructure\TaskExecution\Queue $queue */
    $queue = ServiceRegister::getService(Queue::CLASS_NAME);
    /** @var \CleverReachConfigService $configService */
    $configService = ServiceRegister::getService(Configuration::CLASS_NAME);

    $queue->enqueue(
        $configService->getQueueName(),
        new RefreshUserInfoTask($accessToken)
    );
  }

}
