<?php

use CleverReach\Infrastructure\TaskExecution\QueueItem;

/**
 * Dashboard View Controller.
 *
 * @see template file cleverreach-dashboard.tpl.php
 */
class CleverReachDashboardController extends CleverReachResolveStateController {
  const CURRENT_STATE_CODE = 'dashboard';
  const TEMPLATE = 'cleverreach_dashboard';

  /**
   * Executes POST request method.
   *
   * Prepares data for dashboard screen.
   *
   * @param $form
   *   A keyed array containing the current state of the form.
   *
   * @return array
   *   Array of template vars.
   *
   * @throws Exception
   */
  public function post(array &$form) {
    return $this->get($form);
  }

  /**
   * Executes GET request method.
   *
   * Prepares data for dashboard screen.
   *
   * @param array $form
   *   A keyed array containing the current state of the form.
   *
   * @return array
   *   Array of template vars.
   *
   * @throws Exception
   */
  public function get(array &$form) {
    parent::get($form);

    $userInfo = $this->getConfigService()->getUserInfo();
    $failureParameters = $this->getInitialSyncFailureParameters();
    $url = 'https://' . $userInfo['login_domain'] . CleverReachConfigService::CLEVERREACH_BUILD_EMAIL_URL;

    return array(
      '#recipient_id' => $userInfo['id'],
      '#integration_name' => $this->getConfigService()->getIntegrationName(),
      '#is_built_first_email' => $this->getConfigService()->isFirstEmailBuilt(),
      '#is_initial_sync_failed' => $failureParameters['is_failed'],
      '#initial_sync_failed_message' => $failureParameters['message'],
      '#urls' => array(
        'logo_url' => $this->getThemePath('images/icon_quickstartmailing.svg', TRUE),
        'help_url' => CleverReachConfigService::CLEVERREACH_HELP_URL,
        'built_email_url' => $url,
        'built_first_email_url' => $this->getControllerUrl('build.first.email'),
        'retry_sync_url' => $this->getControllerUrl('retry.sync'),
      ),
      '#theme' => self::TEMPLATE,
      '#attached' => array(
        'css' => array(
          $this->getThemePath('css/cleverreach.css'),
          $this->getThemePath('css/cleverreach-dashboard.css'),
        ),
        'js' => array(
          $this->getThemePath('js/cleverreach.dashboard.js'),
          $this->getThemePath('js/cleverreach.ajax.js'),
        ),
      ),
    );
  }

  /**
   * Gets fail parameters for initial sync task if it failed.
   *
   * @return array
   *   Returns initial sync status array.
   *   Example: ['is_failed' => TRUE, 'message' => ''].
   */
  private function getInitialSyncFailureParameters() {
    $params = array('is_failed' => FALSE, 'message' => '');
    /** @var CleverReach\Infrastructure\TaskExecution\QueueItem $initialSyncTask */
    $initialSyncTask = $this->getQueueService()->findLatestByType('InitialSyncTask');

    if (
        $initialSyncTask &&
        $initialSyncTask->getStatus() === QueueItem::FAILED
    ) {
      $params = array(
        'is_failed' => TRUE,
        'message' => $initialSyncTask->getFailureDescription(),
      );
    }

    return $params;
  }

}
