<?php

/**
 * CleverReach base schema field class.
 */
abstract class CleverReachBaseField {
  /**
   * @var array
   */
  protected $field;

  /**
   * BaseField constructor.
   *
   * @param array $field
   */
  public function __construct($field) {
    $this->field = $field;
  }

  /**
   * @return \CleverReach\BusinessLogic\Utility\ArticleSearch\Schema\SchemaAttribute
   */
  abstract public function getSchemaField();

  /**
   * @param object $node
   *
   * @return \CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\SimpleAttribute
   */
  abstract public function getSearchResultValue($node);

  /**
   * @return bool
   *   Returns true if field is single value, otherwise if multi value field returns false.
   */
  protected function isSingleValue() {
    return (int) $this->field['cardinality'] === 1;
  }

  /**
   * Returns unique machine code of the field.
   *
   * @return string
   *   Field name.
   */
  protected function getName() {
    return isset($this->field['field_name']) ? $this->field['field_name'] : '';
  }

  /**
   * Returns field label.
   *
   * @return string
   *   Label of the field.
   */
  protected function getLabel() {
    return isset($this->field['instance']['label']) ?
        $this->field['instance']['label'] : '';
  }

  /**
   * Gets value for specific node.
   *
   * @param object $node
   *   Content object.
   *
   * @return array
   *   List of values for field.
   */
  protected function getValue($node) {
    $code = $this->getName();
    return isset($node->{$code}['und']) ? $node->{$code}['und'] : array();
  }

  /**
   * Gets list of searchable conditions.
   *
   * @see \CleverReach\BusinessLogic\Utility\ArticleSearch\Conditions
   *
   * @return array
   *   Gets supported searchable conditions.
   */
  protected function getSearchableConditions() {
    return array();
  }

  /**
   * Indicates whether field is searchable or not.
   *
   * @return bool
   *   If field searchable, return true, otherwise false.
   */
  protected function isSearchable() {
    return FALSE;
  }

}
